'---------------------------------------------------------------------
'  Copyright (C) Microsoft Corporation.  All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
' KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
' IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
' PARTICULAR PURPOSE.
'
'---------------------------------------------------------------------

Option Explicit On
Option Strict On

Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.ComponentModel.Design
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Design
Imports System.Reflection
Imports System.Windows.Forms
Imports System.Windows.Forms.Design
Imports System.Windows.Forms.Design.Behavior

Namespace Extensions

    ''' <summary>
    ''' Provides an abstract base implementation for the 
    ''' <see cref="DesignSurfaceExtender"/> component. 
    ''' </summary>
    Public MustInherit Class DesignSurfaceExtension
        Implements IDisposable

        ''' <summary>
        ''' Backs the <see cref="ActionList"/> property.
        ''' </summary>
        Protected actionListValue As DesignerActionList

        Private disposed As Boolean
        Private serviceProviderValue As IServiceProvider
        Private referenceServiceValue As IReferenceService
        Private componentChangeServiceValue As IComponentChangeService
        Private selectionServiceValue As ISelectionService
        Private behaviorServiceValue As BehaviorService

        Private enabledValue As Boolean

        ''' <summary>
        ''' Backs the <see cref="Adorner"/> property.
        ''' </summary>
        Protected adornerValue As Adorner

        ''' <summary>
        ''' Initializes a new instance of the <see cref="DesignSurfaceExtension"/> class.
        ''' </summary>
        ''' <param name="serviceProvider">An <see cref="IServiceProvider"/> which
        ''' provides design-time services.</param>
        Protected Sub New(ByVal serviceProvider As IServiceProvider)
            If serviceProvider IsNot Nothing Then
                Me.serviceProviderValue = serviceProvider
            Else
                Throw New ArgumentNullException("serviceProvider", "must be assigned")
            End If

            Me.CreateAdorner()

            ' Handle the ComponentAdded and ComponentRemoved events.
            AddHandler Me.ComponentChangeService.ComponentAdded, AddressOf ComponentAddedEventHandler
            AddHandler Me.ComponentChangeService.ComponentRemoved, AddressOf ComponentRemovedEventHandler

        End Sub

#Region "Extensibility Points"


        ''' <summary>
        ''' Populates the <see cref="Adorner"/> with the <see cref="ExtensionGlyph"/>
        ''' objects managed by this extension.
        ''' </summary>
        ''' <param name="control">The control to which the <see cref="ExtensionGlyph"/>
        ''' objects will be bound.</param>
        ''' <remarks>This method is not <c>abstract</c> because deriving types
        ''' are not required to override it. For example, extensions which
        ''' provide <see cref="ExtensionGlyph"/> implementations which are not
        ''' bound to particular control instances may not need to implement
        ''' this method.
        ''' </remarks>
        Protected Overridable Sub AddGlyphsForControl(ByVal control As Control)
            Throw New NotImplementedException()
        End Sub

        ''' <summary>
        ''' Removes the <see cref="ExtensionGlyph"/> objects for the given control
        ''' from the related <see cref="Adorner"/>.
        ''' </summary>
        ''' <param name="control">The control to which the <see cref="ExtensionGlyph"/>
        ''' objects will be bound.</param>
        ''' <remarks>This method is not <c>abstract</c> because deriving types
        ''' are not required to override it. For example, extensions which
        ''' provide <see cref="ExtensionGlyph"/> implementations which are not
        ''' bound to particular control instances may not need to implement
        ''' this method.
        ''' </remarks>
        Protected Overridable Sub RemoveGlyphsForControl(ByVal control As Control)
            Throw New NotImplementedException()
        End Sub

        ''' <summary>
        ''' Creates the <see cref="Adorner"/> provided by this 
        ''' extension. 
        ''' </summary>
        ''' <returns>A new <see cref="Adorner"/> populated with type-specific glyphs.</returns>
        Private Function CreateAdorner() As Adorner
            Me.adornerValue = New Adorner()
            Me.adornerValue.Enabled = False

            Me.BehaviorService.Adorners.Add(Me.adornerValue)

            Return adornerValue
        End Function

        ''' <summary>
        ''' Populates the <see cref="System.Windows.Forms.Design.Behavior.Adorner.Glyphs"/> collection.
        ''' </summary>
        ''' <param name="adorner">The <see cref="Adorner"/> to populate with glyphs.</param>
        ''' <remarks>
        ''' <para>
        ''' Deriving classes may provide a type-specific implementation of
        ''' the <see cref="PopulateAdorner"/> method.
        ''' </para>
        ''' <para>
        ''' Implementations are expected to clear the adorner's glyph 
        ''' collection before populating the collection.
        ''' </para>
        ''' </remarks>
        Protected Overridable Sub PopulateAdorner(ByVal adorner As Adorner)
            adorner.Glyphs.Clear()

            Dim controls() As Object
            controls = Me.ReferenceService.GetReferences(GetType(Control))

            For i As Integer = 0 To controls.Length - 1
                Dim c As Control = CType(controls(i), Control)
                Me.AddGlyphsForControl(c)
            Next
        End Sub

        ''' <summary>
        ''' Handles the <see cref="IComponentChangeService.ComponentAdded"/> event. 
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e">A <see cref="ComponentEventArgs"/> which holds the event data.</param>
        ''' <remarks>Deriving classes may provide a type-specific implementation of
        ''' the <see cref="ComponentAddedEventHandler"/> method.
        ''' </remarks>
        Protected Overridable Sub ComponentAddedEventHandler(ByVal sender As Object, ByVal e As ComponentEventArgs)
            If TypeOf e.Component Is Control Then
                If Me.Enabled Then
                    Me.PopulateAdorner(Me.adornerValue)

                    ' IReferenceService workaround.
                    Me.AddGlyphsForControl(CType(e.Component, Control))
                End If
            End If
        End Sub

        ''' <summary>
        ''' Handles the <see cref="IComponentChangeService.ComponentRemoved"/> event. 
        ''' </summary>
        ''' <param name="sender"></param>
        ''' <param name="e">A <see cref="ComponentEventArgs"/> which holds the event data.</param>
        Protected Overridable Sub ComponentRemovedEventHandler(ByVal sender As Object, ByVal e As ComponentEventArgs)
            If TypeOf e.Component Is Control Then
                Me.RemoveGlyphsForControl(CType(e.Component, Control))

                Me.Invalidate()
            End If
        End Sub

#End Region

#Region "IExtension Members"

        ''' <summary>
        ''' Gets or sets a value indicating whether the extension is active.
        ''' </summary>
        Public Property Enabled() As Boolean
            Get
                Return Me.enabledValue
            End Get
            Set(ByVal value As Boolean)
                ' Workaround: DropBehavior enables all adorners
                ' regardless of previous Enabled state, so the
                ' value reported by Adorner.Enabled is unreliable.
                If Me.enabledValue <> value Then
                    Me.enabledValue = value

                    ' Workaround: DropBehavior enables all adorners
                    ' regardless of previous Enabled state.
                    If value Then
                        Me.PopulateAdorner(Me.adornerValue)
                    Else
                        Me.adornerValue.Glyphs.Clear()
                    End If

                    Me.adornerValue.Enabled = Me.enabledValue
                End If
            End Set
        End Property

        ' <summary>
        ' Gets the <see cref="Adorner"/> provided by the extension. 
        ' </summary>
        'Public ReadOnly Property Adorner() As Adorner
        '    Get
        '        If Me.adornerValue Is Nothing Then
        '            Me.adornerValue = DesignSurfaceExtension.CreateAdorner()

        '            Me.PopulateAdorner(Me.adornerValue)

        '            Me.BehaviorService.Adorners.Add(Me.adornerValue)
        '        End If

        '        Return Me.adornerValue
        '    End Get
        'End Property

        Public Sub Invalidate()
            Me.adornerValue.Invalidate()
        End Sub

        ''' <summary>
        ''' Gets the <see cref="DesignerActionList"/> provided by the extension. 
        ''' </summary>
        Public Overridable ReadOnly Property ActionList() As DesignerActionList
            Get
                Return Nothing
            End Get
        End Property

#End Region

#Region "IDisposable Implementation"

        ''' <summary>
        ''' Releases all resources used by the extension. 
        ''' </summary>
        ''' <remarks>The <see cref="DesignSurfaceExtension"/> class implements the 
        ''' the Dispose pattern in order to release event handlers.
        ''' </remarks>
        Public Sub Dispose() Implements IDisposable.Dispose
            Dispose(True)

            GC.SuppressFinalize(Me)
        End Sub

        ''' <summary>
        ''' Releases the unmanaged resources used by the extension and
        ''' optionally releases the managed resources. 
        ''' </summary>
        ''' <param name="disposing"><c>true</c> to release both managed and unmanaged resources; 
        ''' <c>false</c> to release only unmanaged resources. </param>
        ''' <remarks>
        ''' This implementation detaches the <see cref="IComponentChangeService"/>
        ''' event handlers.
        ''' </remarks>
        Protected Overridable Sub Dispose(ByVal disposing As Boolean)
            If Not Me.disposed Then
                If disposing Then
                    If Me.ComponentChangeService Is Nothing Then
                        AddHandler Me.ComponentChangeService.ComponentAdded, AddressOf ComponentAddedEventHandler
                        AddHandler Me.ComponentChangeService.ComponentRemoved, AddressOf ComponentRemovedEventHandler
                    End If
                End If
            End If

            Me.disposed = True
        End Sub

#End Region

#Region "Designer Services"

        ''' <summary>
        ''' Gets the <see cref="IServiceProvider"/> for this extension.
        ''' </summary>
        Protected ReadOnly Property ServiceProvider() As IServiceProvider
            Get
                Return Me.serviceProviderValue
            End Get
        End Property

        ''' <summary>
        ''' Gets a reference to the <see cref="IReferenceService"/>.
        ''' </summary>
        Protected ReadOnly Property ReferenceService() As IReferenceService
            Get
                If Me.referenceServiceValue Is Nothing Then
                    Me.referenceServiceValue = _
                        CType(Me.ServiceProvider.GetService(GetType(IReferenceService)), _
                        IReferenceService)
                End If

                Return Me.referenceServiceValue
            End Get
        End Property

        ''' <summary>
        ''' Gets a reference to the <see cref="IComponentChangeService"/>.
        ''' </summary>
        Protected ReadOnly Property ComponentChangeService() As IComponentChangeService
            Get
                If Me.componentChangeServiceValue Is Nothing Then
                    Me.componentChangeServiceValue = _
                        CType(Me.ServiceProvider.GetService(GetType(IComponentChangeService)), _
                        IComponentChangeService)
                End If

                Return componentChangeServiceValue
            End Get
        End Property

        ''' <summary>
        ''' Gets a reference to the <see cref="ISelectionService"/>.
        ''' </summary>
        Protected ReadOnly Property SelectionService() As ISelectionService
            Get
                If Me.selectionServiceValue Is Nothing Then
                    Me.selectionServiceValue = _
                        CType(Me.ServiceProvider.GetService(GetType(ISelectionService)), _
                        ISelectionService)
                End If

                Return Me.selectionServiceValue
            End Get
        End Property

        ''' <summary>
        ''' Gets a reference to the <see cref="BehaviorService"/>.
        ''' </summary>
        Protected ReadOnly Property BehaviorService() As BehaviorService
            Get
                If Me.behaviorServiceValue Is Nothing Then
                    Me.behaviorServiceValue = _
                        CType(Me.ServiceProvider.GetService(GetType(BehaviorService)), _
                        BehaviorService)
                End If

                Return Me.behaviorServiceValue
            End Get
        End Property

#End Region

    End Class

End Namespace
